<?
if(!function_exists('curl_init') || !function_exists('xmlrpc_encode_request')){
	die('modules curl and xml-rpc are required');
}
class client_xmlrpc {
    /**
     * Art der Kommunikation
     * 
     * vorerst werden nur console und http untersttzt
     *
     * @var string
     */
    private $protocol = 'console';
    /**
     * Generierung des HTTP Headers
     *
     * Gibt an ob der HTTP Header versendet und ausgewertet werden soll.
     *
     * @var boolean
     */
    private $http = null;
    /**
     * Zieladresse des Servers
     *
     * @var string
     */
    private $host = null;
    /**
     * Port auf dem der Server lauscht
     *
     * @var int
     */
    private $port = null;
    /**
     * URL des Servers.
     * 
     * Nur bei http-Protokoll relevant
     *
     * @var string
     */
    private $url = null;
    /**
     * Abfrage die an den Server geschickt wird ( in XML )
     *
     * @var string
     */
    private $request = null;
    /**
     * Antwort was von dem Server ( in XML )
     *
     * @var string
     */
    private $response = null;
    /**
     * Ausgabekodierungsoptionen
     *
     * @var array
     */
    private $request_options = array(
        'encoding' => 'utf-8'
    );
    /**
     * Konstruktor der Klasse
     * 
     * beim gestetzten URL Parameter werden $this->host und $this->port gegebenenfalls berschrieben 
     *
     * @param array $options
     */
    public function __construct (array $options)
    {
        $this->host = @$options['host'];
        $this->port = @$options['port'];
        $this->http = isset($options['http_header']) ? $options['http_header'] : true; 
        // falls URL angegeben wurde, handelt es sich um http-Protokol
        if (isset($options['url']))
        {
            // URL auseinander nehmen
            $url = parse_url($options['url']);
            // Zielurl bestimmen
            $this->url = isset($url['query']) ? $url['path'] . '?' . $url['query'] : $url['path'];
            // Protokoll setzen
            $this->protocol = $url['scheme'];
            // Port setzen, default ist 80
            $this->port = isset($url['port']) ? $url['port'] : 80;
            // Host setzen
            $this->host = $url['host'];
        }
        if (isset($options['request']) && is_array($options['request']))
        {
            $this->request_options = array_merge($this->request_options, $options['request']);
        }
    }
    /**
     * Ruft eine Methode auf dem Server auf
     * 
     * Diese Methode fgt alle Aufrufe in Form von $client->Methode('param1', 'param2)
     * und ruft diese auf dem Server auf, das Ergebnis wird dabei als PHP Variablen zurck geliefert
     *
     * @param string $name	Name der Methode auf dem Zielserver
     * @param array $params Parameter der Methode
     * @return mixed
     */
    public function __call ($name, $params=null)
    {
        $this->request = xmlrpc_encode_request($name, $params, $this->request_options);
        return $this->_do_call();
    }
    /**
     * Hilfsfunktion fr die Aufrufe auf dem Zielserver
     *
     * @return mixed
     */
    private function _do_call ()
    {
        // Fehlermeldungen definieren
        $errno = $errstr = null;
        if(!$this->http)
        {
	        // Verbindung herstellen
	        if (! $fp = @fsockopen($this->host, $this->port, $errno, $errstr))
	        {
	            throw new Exception($errstr, $errno);
	        }
	        // Daten senden
	        if (! fputs($fp, $this->request, strlen($this->request)))
	        {
	            throw new Exception("Write Error");
	        }
	        // Ergebnis lesen
	        $this->response = '';
	        while (! feof($fp))
	        {
	            $this->response .= fgets($fp);
	        }
	        // Verbindung schlieen
	        fclose($fp);
        }
        else
        {
        	// make URL
        	$url = $this->protocol."://".$this->host.":".$this->port.$this->url;
        	// init class
        	$http = new client_http(array('url'=>$url));
        	// Send request
        	$http->send_data($this->request);
        	// get response
        	$this->response=$http->get_data();
        }

        // Daten in die entsprechenden Werte umwandeln
        $data = xmlrpc_decode($this->response);
        // Auf Fehler berprfen
        if (is_array($data) && xmlrpc_is_fault($data))
        {
            throw new Exception($data['faultString'], $data['faultCode']);
        }
        // Alles ausgeben
        return $data;
    }
    /**
     * Gibt die Liste aller Methoden die auf dem Server existieren
     *
     * @return array
     * @todo die Methode muss auf dem Server implementiert werden
     */
    public function __getFunctions ()
    {
        return $this->__call('system.listMethods');
    }
    /**
     * Gibt die letzte Abfrage aus
     *
     * @return string
     */
    public function __getLastRequest ()
    {
        return $this->request;
    }
    /**
     * Gibt das letzte Antwort aus
     *
     * @return string
     */
    public function __getLastResponse ()
    {
        return $this->response;
    }
}
class client_json {
    /**
     * Art der Kommunikation
     * 
     * vorerst werden nur http untersttzt
     *
     * @var string
     */
    private $protocol = 'http';
    /**
     * Zieladresse des Servers
     *
     * @var string
     */
    private $host = null;
    /**
     * Port auf dem der Server lauscht
     *
     * @var int
     */
    private $port = 80;
    /**
     * API-URL der Homematic.
     * 
     *
     * @var string
     */
    private $url = '/api/homematic.cgi';
    /**
     * Abfrage die an den Server geschickt wird ( in JSON )
     *
     * @var string
     */
    private $request = null;
    /**
     * Antwort was von dem Server ( in JSON )
     *
     * @var string
     */
    private $response = null;
    /**
     * Konstruktor der Klasse
     * 
     * beim gestetzten URL Parameter werden $this->host und $this->port gegebenenfalls berschrieben 
     *
     * @param array $options
     */
    public function __construct (array $options)
    {
        $this->host = @$options['host'];
        $this->port = isset($options['port']) ? $options['port'] : 80 ;
        // falls URL angegeben wurde, diese parsen und host/port ermitteln
        if (isset($options['url']))
        {
            // URL auseinander nehmen
            $url = parse_url($options['url']);
            // Zielurl bestimmen
            $this->url = isset($url['query']) ? $url['path'] . '?' . $url['query'] : $url['path'];
            // Protokoll setzen
            $this->protocol = $url['scheme'];
            // Port setzen, default ist 80
            $this->port = isset($url['port']) ? $url['port'] : 80;
            // Host setzen
            $this->host = $url['host'];
        }
	}
    /**
     * Ruft eine Methode auf dem Server auf
     * 
     * Diese Methode fgt alle Aufrufe in Form von $client->Methode('param1', 'param2)
     * und ruft diese auf dem Server auf, das Ergebnis wird dabei als PHP Variablen zurck geliefert
     *
     * @param string $name	Name der Methode auf dem Zielserver
     * @param array $params Parameter der Methode
     * @return array
     */
    public function __call ($name, array $params=array())
    {
    	// Nachricht generieren
    	$message = array('method'=>$name, 'params'=> $params[0]);
    	// Nachricht kodieren
        $this->request = json_encode($message);
        // Nachricht senden
        return $this->_do_call();
    }
    /**
     * Hilfsfunktion fr die Aufrufe auf dem Zielserver
     *
     * @todo Errors abfangen
     * @return array
     */
    private function _do_call ()
    {
        // Fehlermeldungen definieren
        $errno = $errstr = null;
        // make URL
        $url = $this->protocol."://".$this->host.":".$this->port.$this->url;
        // init class
        $http = new client_http(array('url'=>$url));
        // Send request
        $http->send_data($this->request);
        // get response
        $this->response=$http->get_data();

        // Daten in die entsprechenden Werte umwandeln
        $data = json_decode($this->response);
        // Auf Fehler berprfen
        if (json_last_error())
        {
            throw new Exception($data->error);
        }
        if(isset($data->error->name))
        {
        	throw new Exception($data->error->message, $data->error->code);
        }
        // Alles ausgeben
        return $data;
    }
    /**
     * Gibt die Liste aller Methoden die auf dem Server existieren
     *
     * @return array
     */
    public function __getFunctions ()
    {
        return $this->__call('system.listMethods');
    }
    /**
     * Gibt die letzte Abfrage aus
     *
     * @return string
     */
    public function __getLastRequest ()
    {
        return $this->request;
    }
    /**
     * Gibt das letzte Antwort aus
     *
     * @return string
     */
    public function __getLastResponse ()
    {
        return $this->response;
    }
}
class client_http {
	/**
	 * Verbindungsidentifer zu dem Server
	 *
	 * @var ressource
	 */
	private $connect = null;
	/**
	 * Ausgabe von dem Server
	 * 
	 * @var string
	 */
	private $result = null;
	/**
	 * Konstruktor der Klasse
	 *
	 *
	 * @param array $options
	 * @todo extra SET_OPT Parameter hier definieren
	 */
	public function __construct (array $options)
	{
		if (isset($options['url']))
		{
			$this->connect = curl_init($options['url']);
		}
		else
		{
			throw new Exception('url parameter is required');
		}

		if(isset($options['methode']) && $options['methode']=='POST')
		{
			curl_setopt($this->connect, CURLOPT_POST, 1);
		}
	}

	/**
	 * Sendet Daten an den Server und parst den Output
	 * 
	 * @param string $data      Daten die gesendet sollen, bei GET var=value&var=value
	 * @param string $method    Methode, aktuell werden POST/GET untersttzt
	 * @param boolean $output   Gibt an, ob die Ausgabe geparst werden soll
	 * 
	 * @return array
	 */
	public function send_data($data, $method='POST', $output=true)
	{
		// Methodenparameter
		switch ($method)
		{
			case 'POST':
				curl_setopt($this->connect, CURLOPT_POST, 1);
				curl_setopt($this->connect, CURLOPT_POSTFIELDS, $data);	
				break;				
		}
		// Outputrckgabe
		if($output)
		{
			curl_setopt($this->connect, CURLOPT_RETURNTRANSFER, true);
		}
		//debuging
		#curl_setopt($this->connect, CURLOPT_VERBOSE, 1);
		// Request senden
		$this->result = curl_exec($this->connect);
		// Verbindung Schliessen
		curl_close($this->connect);
	}
	/**
	 * Gibt die Ausgabe der Anfrage zurck
	 * 
	 * @return array
	 */
	public function get_data()
	{	
	    if(!is_null($this->result))
	    {
		    return $this->result;	
	    }
	}
}
?>

